@@ -34,8 +34,8 @@
 
 #define MAKE_NV(K, V)                                                          \
   {                                                                            \
-    (uint8_t *)K, (uint8_t *)V, sizeof(K) - 1, sizeof(V) - 1,                  \
-        NGHTTP2_NV_FLAG_NONE                                                   \
+      (uint8_t *)K,  (uint8_t *)V,         sizeof(K) - 1,                      \
+      sizeof(V) - 1, NGHTTP2_NV_FLAG_NONE,                                     \
   }
 
 static void deflate(nghttp2_hd_deflater *deflater,
