@@ -24,7 +24,14 @@
  */
 #include "shrpx_http_downstream_connection.h"
 
-#include <openssl/rand.h>
+#include "ssl_compat.h"
+
+#ifdef NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <wolfssl/options.h>
+#  include <wolfssl/openssl/rand.h>
+#else // !NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <openssl/rand.h>
+#endif // !NGHTTP2_OPENSSL_IS_WOLFSSL
 
 #include "shrpx_client_handler.h"
 #include "shrpx_upstream.h"
@@ -41,7 +48,6 @@
 #include "shrpx_log.h"
 #include "http2.h"
 #include "util.h"
-#include "ssl_compat.h"
 
 using namespace nghttp2;
 
@@ -266,13 +272,26 @@ constexpr llhttp_settings_t htp_hooks = {
     htp_msg_begincb,     // llhttp_cb      on_message_begin;
     nullptr,             // llhttp_data_cb on_url;
     nullptr,             // llhttp_data_cb on_status;
+    nullptr,             // llhttp_data_cb on_method;
+    nullptr,             // llhttp_data_cb on_version;
     htp_hdr_keycb,       // llhttp_data_cb on_header_field;
     htp_hdr_valcb,       // llhttp_data_cb on_header_value;
+    nullptr,             // llhttp_data_cb on_chunk_extension_name;
+    nullptr,             // llhttp_data_cb on_chunk_extension_value;
     htp_hdrs_completecb, // llhttp_cb      on_headers_complete;
     htp_bodycb,          // llhttp_data_cb on_body;
     htp_msg_completecb,  // llhttp_cb      on_message_complete;
-    nullptr,             // llhttp_cb      on_chunk_header
-    nullptr,             // llhttp_cb      on_chunk_complete
+    nullptr,             // llhttp_cb      on_url_complete;
+    nullptr,             // llhttp_cb      on_status_complete;
+    nullptr,             // llhttp_cb      on_method_complete;
+    nullptr,             // llhttp_cb      on_version_complete;
+    nullptr,             // llhttp_cb      on_header_field_complete;
+    nullptr,             // llhttp_cb      on_header_value_complete;
+    nullptr,             // llhttp_cb      on_chunk_extension_name_complete;
+    nullptr,             // llhttp_cb      on_chunk_extension_value_complete;
+    nullptr,             // llhttp_cb      on_chunk_header;
+    nullptr,             // llhttp_cb      on_chunk_complete;
+    nullptr,             // llhttp_cb      on_reset;
 };
 } // namespace
 
@@ -412,8 +431,8 @@ int HttpDownstreamConnection::initiate_connection() {
 
       auto sni_name =
           addr_->sni.empty() ? StringRef{addr_->host} : StringRef{addr_->sni};
-      if (!util::numeric_host(sni_name.c_str())) {
-        SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name.c_str());
+      if (!util::numeric_host(sni_name.data())) {
+        SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name.data());
       }
 
       auto session = tls::reuse_tls_session(addr_->tls_session_cache);
@@ -437,7 +456,7 @@ int HttpDownstreamConnection::initiate_connection() {
     ev_set_cb(&conn_.rt, timeoutcb);
     if (conn_.read_timeout < group_->shared_addr->timeout.read) {
       conn_.read_timeout = group_->shared_addr->timeout.read;
-      conn_.last_read = ev_now(conn_.loop);
+      conn_.last_read = std::chrono::steady_clock::now();
     } else {
       conn_.again_rt(group_->shared_addr->timeout.read);
     }
@@ -551,9 +570,10 @@ int HttpDownstreamConnection::push_request_headers() {
         return -1;
       }
       auto iov = make_byte_ref(balloc, base64::encode_length(nonce.size()) + 1);
-      auto p = base64::encode(std::begin(nonce), std::end(nonce), iov.base);
+      auto p =
+          base64::encode(std::begin(nonce), std::end(nonce), std::begin(iov));
       *p = '\0';
-      auto key = StringRef{iov.base, p};
+      auto key = StringRef{std::span{std::begin(iov), p}};
       downstream_->set_ws_key(key);
 
       buf->append("Sec-Websocket-Key: ");
@@ -583,13 +603,16 @@ int HttpDownstreamConnection::push_request_headers() {
   auto upstream = downstream_->get_upstream();
   auto handler = upstream->get_client_handler();
 
-#if OPENSSL_1_1_1_API
+#if defined(NGHTTP2_GENUINE_OPENSSL) ||                                        \
+    defined(NGHTTP2_OPENSSL_IS_BORINGSSL) ||                                   \
+    defined(NGHTTP2_OPENSSL_IS_WOLFSSL)
   auto conn = handler->get_connection();
 
   if (conn->tls.ssl && !SSL_is_init_finished(conn->tls.ssl)) {
     buf->append("Early-Data: 1\r\n");
   }
-#endif // OPENSSL_1_1_1_API
+#endif // NGHTTP2_GENUINE_OPENSSL || NGHTTP2_OPENSSL_IS_BORINGSSL ||
+       // NGHTTP2_OPENSSL_IS_WOLFSSL
 
   auto fwd =
       fwdconf.strip_incoming ? nullptr : req.fs.header(http2::HD_FORWARDED);
@@ -862,7 +885,7 @@ void HttpDownstreamConnection::detach_downstream(Downstream *downstream) {
   ev_set_cb(&conn_.rt, idle_timeoutcb);
   if (conn_.read_timeout < downstreamconf.timeout.idle_read) {
     conn_.read_timeout = downstreamconf.timeout.idle_read;
-    conn_.last_read = ev_now(conn_.loop);
+    conn_.last_read = std::chrono::steady_clock::now();
   } else {
     conn_.again_rt(downstreamconf.timeout.idle_read);
   }
@@ -912,6 +935,17 @@ int htp_hdrs_completecb(llhttp_t *htp) {
   auto &resp = downstream->response();
   int rv;
 
+  auto &balloc = downstream->get_block_allocator();
+
+  for (auto &kv : resp.fs.headers()) {
+    kv.value = util::rstrip(balloc, kv.value);
+
+    if (kv.token == http2::HD_TRANSFER_ENCODING &&
+        !http2::check_transfer_encoding(kv.value)) {
+      return -1;
+    }
+  }
+
   auto config = get_config();
   auto &loggingconf = config->logging;
 
@@ -985,6 +1019,16 @@ int htp_hdrs_completecb(llhttp_t *htp) {
   resp.connection_close = !llhttp_should_keep_alive(htp);
   downstream->set_response_state(DownstreamState::HEADER_COMPLETE);
   downstream->inspect_http1_response();
+
+  if (htp->flags & F_CHUNKED) {
+    downstream->set_chunked_response(true);
+  }
+
+  auto transfer_encoding = resp.fs.header(http2::HD_TRANSFER_ENCODING);
+  if (transfer_encoding && !downstream->get_chunked_response()) {
+    resp.connection_close = true;
+  }
+
   if (downstream->get_upgraded()) {
     // content-length must be ignored for upgraded connection.
     resp.fs.content_length = -1;
@@ -1138,6 +1182,12 @@ int htp_bodycb(llhttp_t *htp, const char *data, size_t len) {
 namespace {
 int htp_msg_completecb(llhttp_t *htp) {
   auto downstream = static_cast<Downstream *>(htp->data);
+  auto &resp = downstream->response();
+  auto &balloc = downstream->get_block_allocator();
+
+  for (auto &kv : resp.fs.trailers()) {
+    kv.value = util::rstrip(balloc, kv.value);
+  }
 
   // llhttp does not treat "200 connection established" response
   // against CONNECT request, and in that case, this function is not
@@ -1206,7 +1256,7 @@ int HttpDownstreamConnection::write_first() {
 }
 
 int HttpDownstreamConnection::read_clear() {
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   std::array<uint8_t, 16_k> buf;
   int rv;
@@ -1245,7 +1295,7 @@ int HttpDownstreamConnection::read_clear() {
 }
 
 int HttpDownstreamConnection::write_clear() {
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   auto upstream = downstream_->get_upstream();
   auto input = downstream_->get_request_buf();
@@ -1293,7 +1343,7 @@ int HttpDownstreamConnection::write_clear() {
 int HttpDownstreamConnection::tls_handshake() {
   ERR_clear_error();
 
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   auto rv = conn_.tls_handshake();
   if (rv == SHRPX_ERR_INPROGRESS) {
@@ -1335,7 +1385,7 @@ int HttpDownstreamConnection::tls_handshake() {
 }
 
 int HttpDownstreamConnection::read_tls() {
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   ERR_clear_error();
 
@@ -1376,7 +1426,7 @@ int HttpDownstreamConnection::read_tls() {
 }
 
 int HttpDownstreamConnection::write_tls() {
-  conn_.last_read = ev_now(conn_.loop);
+  conn_.last_read = std::chrono::steady_clock::now();
 
   ERR_clear_error();
 
