@@ -31,9 +31,6 @@
 #include <iostream>
 #include <fstream>
 
-#include <openssl/crypto.h>
-#include <openssl/conf.h>
-
 #ifdef HAVE_LIBBROTLI
 #  include <brotli/encode.h>
 #  include <brotli/decode.h>
@@ -41,6 +38,15 @@
 
 #include "ssl_compat.h"
 
+#ifdef NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <wolfssl/options.h>
+#  include <wolfssl/openssl/crypto.h>
+#  include <wolfssl/openssl/conf.h>
+#else // !NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <openssl/crypto.h>
+#  include <openssl/conf.h>
+#endif // !NGHTTP2_OPENSSL_IS_WOLFSSL
+
 namespace nghttp2 {
 
 namespace tls {
@@ -178,6 +184,10 @@ int cert_decompress(SSL *ssl, CRYPTO_BUFFER **out, size_t uncompressed_len,
 }
 #endif // NGHTTP2_OPENSSL_IS_BORINGSSL && HAVE_LIBBROTLI
 
+#if defined(NGHTTP2_GENUINE_OPENSSL) ||                                        \
+    defined(NGHTTP2_OPENSSL_IS_BORINGSSL) ||                                   \
+    defined(NGHTTP2_OPENSSL_IS_LIBRESSL) ||                                    \
+    (defined(NGHTTP2_OPENSSL_IS_WOLFSSL) && defined(HAVE_SECRET_CALLBACK))
 namespace {
 std::ofstream keylog_file;
 
@@ -203,6 +213,13 @@ int setup_keylog_callback(SSL_CTX *ssl_ctx) {
 
   return 0;
 }
+#else  // !NGHTTP2_GENUINE_OPENSSL && !NGHTTP2_OPENSSL_IS_BORINGSSL &&
+       // !NGHTTP2_OPENSSL_IS_LIBRESSL && !(NGHTTP2_OPENSSL_IS_WOLFSSL &&
+       // HAVE_SECRET_CALLBACK)
+int setup_keylog_callback(SSL_CTX *ssl_ctx) { return 0; }
+#endif // !NGHTTP2_GENUINE_OPENSSL && !NGHTTP2_OPENSSL_IS_BORINGSSL &&
+       // !NGHTTP2_OPENSSL_IS_LIBRESSL && !(NGHTTP2_OPENSSL_IS_WOLFSSL &&
+       // HAVE_SECRET_CALLBACK)
 
 } // namespace tls
 
